/*
 * A flexpage mapping for accessors.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/semaphore.h>
#include <l4/sys/types.h>

#include <stdio.h>

#include <map>
#include <set>

#include <fsserver/flexpage.h>

/* Forward declarations. */

class Pages;
class PagingAccessor;



/* Collections of flexpages ordered by file positions. */

typedef std::map<unsigned long, Flexpage *> Flexpage_map;
typedef std::pair<unsigned long, Flexpage *> Flexpage_map_item;



/* Page map for accessors. */

class AccessMap
{
protected:
  /* Available flexpages within each file ordered by file position. */

  Flexpage_map _map;
  l4_cap_idx_t _semaphore;

  void lock_map()
  {
    l4_semaphore_down(_semaphore, L4_IPC_NEVER);
  }

  void unlock_map()
  {
    l4_semaphore_up(_semaphore);
  }

  Flexpage *_find(unsigned long data_position);

public:
  explicit AccessMap();

  virtual ~AccessMap();

  Flexpage *find(unsigned long data_position);

  void flush_all(unsigned long start, unsigned long end, bool keep,
                 PagingAccessor *accessor, Pages *pages);

  void insert(Flexpage *flexpage);

  bool remove(Flexpage *flexpage);

  void purge(PagingAccessor *accessor, Pages *pages);
};
