/*
 * An opener context resource.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/opener_support.h>
#include <fsserver/pager_resource.h>

/* Implemented interfaces. */

#include "opener_context_object_interface.h"



/* An opener resource acting as a simple pager object, offering a selection of
   operations employing the contents of the dataspace as the path of a file or
   resource.

   The act of accessing or opening a file or resource is delegated to an actual
   opener (such as a filesystem). */

class OpenerContextResource : public PagerResource, public OpenerContextObject
{
protected:
  OpenerSupport *_opener;

public:
  OpenerContextResource(void *buffer, unsigned long buffer_size,
                        OpenerSupport *opener)
  : PagerResource(buffer, buffer_size), _opener(opener)
  {
  }

  /* Open the resource with no defined buffer or opener. */

  OpenerContextResource()
  : PagerResource(NULL, 0), _opener(NULL)
  {
  }

  /* Server details. */

  int expected_items();

  ipc_server_handler_type handler();

  void *interface()
  { return static_cast<OpenerContextObject *>(this); }

  /* Dataspace operations. */

  virtual long map(unsigned long offset, l4_addr_t hot_spot,
                   unsigned long flags, l4_snd_fpage_t *region)
  { return PagerResource::map(offset, hot_spot, flags, region); }

  virtual long info(unsigned long *size, unsigned long *flags)
  { return PagerResource::info(size, flags); }

  /* OpenerContext operations. */

  virtual long getfs(l4_cap_idx_t *fs);

  virtual long mount(l4_cap_idx_t fs);

  virtual long open(int flags, size_t *size, l4_cap_idx_t *file);
};
