/*
 * A simple pager abstraction for handling map requests.
 *
 * Copyright (C) 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>



/* A pager exporting a preallocated memory region. */

class SimplePager
{
  void *_buffer;
  unsigned long _buffer_size;

public:
  SimplePager(void *buffer, unsigned long buffer_size)
  : _buffer(buffer), _buffer_size(buffer_size)
  {
  }

  /* Deallocation of resources. */

  virtual void close();

  /* Access methods. */

  void *buffer() { return _buffer; }
  unsigned long size() { return _buffer_size; }

  /* Operation methods. */

  long map(unsigned long offset, l4_addr_t hot_spot, unsigned long flags,
           l4_snd_fpage_t *region);

  long info(unsigned long *size, unsigned long *flags);
};
