/*
 * A "stat" (metadata) resource.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/pager_resource.h>

/* Implemented interfaces. */

#include <fsserver/stat_object_interface.h>



/* A "stat" (metadata) resource acting as a simple pager object, providing
   access to metadata content via a dataspace interface.

   All operations are delegated to base classes, with this class merely
   providing the external interface. */

class StatResource : public PagerResource, public StatObject
{
public:
  StatResource(void *buffer, unsigned long buffer_size)
  : PagerResource(buffer, buffer_size)
  {
  }

  /* Server details. */

  int expected_items();

  ipc_server_handler_type handler();

  void *interface()
  { return static_cast<StatObject *>(this); }

  /* Dataspace operations. */

  virtual long map(unsigned long offset, l4_addr_t hot_spot,
                   unsigned long flags, l4_snd_fpage_t *region)
  { return PagerResource::map(offset, hot_spot, flags, region); }

  virtual long info(unsigned long *size, unsigned long *flags)
  { return PagerResource::info(size, flags); }
};
